# -*- coding: utf-8 -*-

import os
import threading
import time
import itertools
from collections import namedtuple

from tensorboard.backend import application
from tensorboard.plugins.audio import audio_plugin
from tensorboard.plugins.core import core_plugin
from tensorboard.plugins.distribution import distributions_plugin
from tensorboard.plugins.graph import graphs_plugin
from tensorboard.plugins.histogram import histograms_plugin
from tensorboard.plugins.image import images_plugin
from tensorboard.plugins.profile import profile_plugin
from tensorboard.plugins.projector import projector_plugin
from tensorboard.plugins.scalar import scalars_plugin
from tensorboard.plugins.text import text_plugin

from .handlers import notebook_dir

TensorBoardInstance = namedtuple(
    'TensorBoardInstance', ['name', 'logdir', 'tb_app', 'thread'])


def start_reloading_multiplexer(multiplexer, path_to_run, reload_interval):
    def _ReloadForever():
        current_thread = threading.currentThread()
        current_thread.stop = False
        current_thread.reload_time = None
        while not current_thread.stop:
            application.reload_multiplexer(multiplexer, path_to_run)
            current_thread.reload_time = time.time()
            time.sleep(reload_interval)
    thread = threading.Thread(target=_ReloadForever)
    thread.daemon = True
    thread.start()
    return thread


def TensorBoardWSGIApp(logdir, plugins, multiplexer, reload_interval):
    path_to_run = application.parse_event_files_spec(logdir)
    if reload_interval:
        thread = start_reloading_multiplexer(
            multiplexer, path_to_run, reload_interval)
    else:
        application.reload_multiplexer(multiplexer, path_to_run)
        thread = None
    tb_app = application.TensorBoardWSGI(plugins)
    manager.add_instance(logdir, tb_app, thread)
    return tb_app


application.TensorBoardWSGIApp = TensorBoardWSGIApp


class TensorboardManger(dict):

    def __init__(self):
        self._logdir_dict = {}

    def _next_available_name(self):
        for n in itertools.count(start=1):
            name = "%d" % n
            if name not in self:
                return name

    def new_instance(self, logdir, reload_interval):
        if not os.path.isabs(logdir) and notebook_dir:
            logdir = os.path.join(notebook_dir, logdir)

        if logdir not in self._logdir_dict:
            purge_orphaned_data = True
            plugins = [
                core_plugin.CorePlugin,
                scalars_plugin.ScalarsPlugin,
                images_plugin.ImagesPlugin,
                audio_plugin.AudioPlugin,
                graphs_plugin.GraphsPlugin,
                distributions_plugin.DistributionsPlugin,
                histograms_plugin.HistogramsPlugin,
                projector_plugin.ProjectorPlugin,
                text_plugin.TextPlugin,
                profile_plugin.ProfilePlugin,
            ]
            reload_interval = reload_interval or 30
            application.standard_tensorboard_wsgi(
                logdir=logdir, reload_interval=reload_interval,
                purge_orphaned_data=purge_orphaned_data, plugins=plugins)

        return self._logdir_dict[logdir]

    def add_instance(self, logdir, tb_application, thread):
        name = self._next_available_name()
        instance = TensorBoardInstance(name, logdir, tb_application, thread)
        self[name] = instance
        self._logdir_dict[logdir] = instance

    def terminate(self, name, force=True):
        if name in self:
            instance = self[name]
            if instance.thread is not None:
                instance.thread.stop = True
            del self[name], self._logdir_dict[instance.logdir]
        else:
            raise Exception("There's no tensorboard instance named %s" % name)


manager = TensorboardManger()



# -*- coding: utf-8 -*-

from .handlers import load_jupyter_server_extension   # noqa

__version__ = "0.1.3.dev0"


def _jupyter_nbextension_paths():
    name = __name__
    section = "tree"
    src = "static"
    return [dict(
        section=section,
        src=src,
        dest=name,
        require="%s/%s" % (name, section))]


def _jupyter_server_extension_paths():
    return [{
        "module": __name__
    }]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import io
import re
from setuptools import setup
from setuptools.command.install import install
from setuptools.command.develop import develop


def read(*names, **kwargs):
    with io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ) as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def enable_extension_after_install():
    from jupyter_tensorboard.application import (
        EnableJupyterTensorboardApp,
    )
    EnableJupyterTensorboardApp.launch_instance(argv=[])


class EnableExtensionDevelop(develop):

    def run(self):
        develop.run(self)
        enable_extension_after_install()


class EnableExtensionInstall(install):

    def run(self):
        install.run(self)
        enable_extension_after_install()


name = 'jupyter_tensorboard'

setup(
    name=name,
    version=find_version(name, '__init__.py'),
    author='lspvic',
    author_email='lspvic@qq.com',
    url='http://github.com/lspvic/jupyter_tensorboard',
    license='MIT License',
    description=(
        'Start tensorboard in Jupyter! '
        'Jupyter notebook integration for tensorboard.'),
    long_description=read("README.rst"),
    keywords=['Jupyter', 'Notebook', 'Tensorboard', 'Tensorflow', ],
    packages=[name],
    package_data={name: ["static/*"]},
    platforms="Linux, Mac OS X, Windows",
    entry_points={
        'console_scripts': [
            'jupyter-tensorboard = jupyter_tensorboard.application:main',
         ],
    },
    scripts=[os.path.join('scripts', p) for p in [
        'jupyter-tensorboard',
    ]],
    install_requires=[
        'notebook>=5.0',
        'tensorflow-tensorboard>=0.1.5',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    cmdclass={
        'install': EnableExtensionInstall,
        'develop': EnableExtensionDevelop,
    },
)
